;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MODULE     : mmx-mode.el
;; DESCRIPTION: A Mathemagix mode for emacs
;; COPYRIGHT  : (C) 2004  Enrique A. Tobis (etobis@dc.uba.ar)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Usage: put the following code in your .emacs file after setting the path
;; (setq auto-mode-alist
;;      (append auto-mode-alist
;;       '(("\\.mmx" . mmx-mode)
;; ("\\.mmh" . mmx-mode))))
;; (autoload 'mmx-mode
;;   "[the correct path]/mmx-mode.el"
;;   "Major mode for editing Mathemagix files" t)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; This software falls under the GNU general public license and comes
;; WITHOUT ANY WARRANTY WHATSOEVER. See the file LICENSE for more details.
;; If you don't have this file, write to the Free Software Foundation, Inc.,
;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Functionality provided:
;; - Keyword Highlighting
;; - System types highlighting
;; - //-style comments highlighting

;; TODO:

;; Functions
;; ---------

;; cons, entries, pretty, reverse

;; New functions are declared using the syntax
;; f(...)...(...): ... == (or :=) ...
;; f: (...) -> ... == (or :=) ...
;; Function declarations maybe preceded by
;; forall(...)
;; exists(...)
;; but these modifiers maybe required to be on a previous line
;; for the highlighting to work.

;; Constants
;; ---------

;; Literal integers, floats and strings, as well as
;; true, false, nil, mmout, mmerr, mmin
;; We also support multiple line strings of the form /"..."/
;; which may include " characters (but not "/).

;; Comments
;; --------

;; recursive multiple-line comments
;; of the form /{ ... }/

;; Indentation
;; -----------

;; The default indentation is 2 spaces and mostly increases
;; inside { ... } blocks or after trailing keywords.

;; Administrative stuff
(defconst mmx-version "0.0.2"
  "mathemagix-mode version number")
(defun mmx-version ()
  "Echo the current version of mathemagix-mode in the minibuffer."
  (interactive)
  (message "mmx-mode version %s" mmx-version)
)

;; We define the hook for this major-mode, so that people can add
;; initialization code
(defvar mmx-mode-hook nil)

;; Customization

;; This function takes a list of strings, representing words, and returns
;; a list or regular expressions which denote those keywords, only when they
;; stand alone (i.e if for is a keyword, it will not be highlighted in before

(defun mmx-stand-alone-keywords (keywords)
  (let ((worklist keywords) (result))
    (while (not (null worklist))
      (setq result (cons (concat "\\<" (car worklist) "\\>") result))
      (setq worklist (cdr worklist))
      )
    (reverse result)
    )
  )

;; This is the list of Mathemagix keywords. You may add or remove any of them
(defvar mmx-keywords 
  (list "and" "begin" "break" "case" "continue" "else"
	"error" "exists" "export" "forall" "for"
	"generate" "if" "import" "join" "lambda"
	"load" "loop" "literal" "obtain" "operator"
	"or" "postfix" "retrieve" "return" "split" "step"
	"until" "use" "while" "yield")
  )

;; This is the list of Mathemagix types. You may add or remove any of them
(defvar mmx-types 
  (list "Alias" "Bool" "Class" "Cross" "Generator" "Int" "List"
	"Literal_Float" "Literal_Int" "Literal_String" "OStream" "String"
	"Table" "Tuple" "VCross" "VTuple"
	))

(defvar mmx-reserved-words
  (mmx-stand-alone-keywords (append mmx-types mmx-keywords)))

;; This defines // style comments
(defvar mmx-comments
  (list '("//.*" . font-lock-comment-face)
	'("/{\\([^/]\\|\\\\.\\)*}/" . font-lock-comment-face)))

(defun mmx-font-lock ()
  (let (mmx-stuff)
    (setq mmx-stuff (append mmx-comments mmx-reserved-words))
    (setq font-lock-defaults '(mmx-stuff nil nil nil))
    (turn-on-font-lock)
    )
  )

;; This is the main function.
(defun mmx-mode ()
  (interactive)
  (kill-all-local-variables)
  (setq major-mode 'mmx-mode) 
  (setq mode-name "Mathemagix") ;This sets the label in the Mode Line
  (mmx-font-lock)
  (run-hooks 'mmx-mode-hook)
  )

;; We make ourselves available
(provide 'mmx-mode)
